/* callbacks.c - controls GTK callback functions (excluding network) */

#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>

#include <gnome.h>

#include <ctype.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "queue.h"
#include "download.h"
#include "servers.h"
#include "chan.h"
#include "xtext.h"
#include "text.h"
#include "utils.h"
//#include "menus.h"

#include "config.h"

#include "debug.h"

extern ConnInfo conn_info;
extern UserInfo user_info;
extern GnapsterMain gnapster_main;
extern SearchTab search_tab;
extern BrowseTab browse_tab;
extern DownloadTab download_tab;
extern UploadTab upload_tab;
extern ConsoleTab console_tab;
extern MOTDTab motd_tab;
extern PropDlg prop_dlg;
extern BuildDlg build_dlg;
extern BrowseDlg browse_dlg;
extern AddDlg add_dlg;
extern EditDlg edit_dlg;
extern ListChanDlg list_chan_dlg;
extern ResumeDlg resume_dlg;
extern RefreshDlg refresh_dlg;

extern int building;
extern GList *total_list;
extern GList *ignore_list;

#define MAX_HISTORY 15

int loc_curr = 0, loc_end = 0;
GList *console_history = NULL;

void gnapster_exit_cb() {
   int w = -1, h = -1;

   if (conn_info.connected) {
      close(conn_info.sock); 
      gdk_input_remove(conn_info.sock_input);
   }

   gdk_window_get_size(gnapster_main.window->window, &w, &h);
   gnome_config_set_int("/gnapster/Geometry/w", w);
   gnome_config_set_int("/gnapster/Geometry/h", h);
   gnome_config_sync();
   
   gtk_main_quit();
}

void safe_exit_check() {
   if (!conn_info.safe_exit)
     return;
   
   if (!GTK_CLIST(upload_tab.clist)->rows &&
       !GTK_CLIST(download_tab.clist)->rows &&
       !GTK_CLIST(download_tab.queue_clist)->rows)
     gnapster_exit_cb();
}

void safe_exit_cb() {
   disconnect_cb();
   
   gnapster_update_status("Safe exit mode activated...finishing all pending transfers", 0);
   conn_info.safe_exit = 1;
   
   safe_exit_check();
}

/* as you can see i read XChat source religiously hehe */

void browse_font_ok_cb(GtkWidget *w, GtkWidget *dlg) {
   char *font;
   
   font = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dlg));
   
   if (prop_dlg.font_entry && font && strlen(font))
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.font_entry), font);
   
   gtk_widget_destroy(dlg);
}

void browse_font_cancel_cb(GtkWidget *w, GtkWidget *dlg) {
   gtk_widget_destroy(dlg);
}

void browse_font_cb() {
   GtkWidget *dlg;
   
   dlg = gtk_font_selection_dialog_new(_("Select Font"));
   
   gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(dlg)->ok_button), "clicked",
		      GTK_SIGNAL_FUNC(browse_font_ok_cb), dlg);
   gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(dlg)->cancel_button), "clicked",
		      GTK_SIGNAL_FUNC(browse_font_cancel_cb), dlg);
   
   gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dlg),
					   gtk_entry_get_text(GTK_ENTRY(prop_dlg.font_entry)));
   
   gtk_widget_show(dlg);
}

void update_shares_cb() {
   shared_list_handle(user_info.upload_dir, UPDATE_LIST);   
}

void connect_last_cb() {
   if (conn_info.connected || conn_info.connecting)
     already_connected(NULL);
   else
     connect_cb();
}

void already_connected_cb(gint reply, gpointer data) {
   if (reply == GNOME_YES) {
      disconnect_cb();
      if (data) {
	 Server *nsrv;
	 
	 nsrv = (Server *)data;
	 browse_dlg_connect_cb(NULL, nsrv);
      } else
	connect_cb();
   } else if (reply != GNOME_NO) {
      g_assert_not_reached();
   } 
}

void already_connected(Server *nsrv) {
   GtkWidget *dialog;
   
   dialog = gnome_question_dialog_parented(_("You're already connected to a server. Disconnect?"),
					   already_connected_cb,
					   nsrv,
					   GTK_WINDOW(gnapster_main.window));
   
   gtk_widget_show(dialog);
}

void connect_official_cb() {
   gnome_config_set_string("/gnapster/Options/server", SERVER_NAPSTER_COM);
   gnome_config_set_int("/gnapster/Options/server_port", 8875);
   gnome_config_sync();
   if (conn_info.connected || conn_info.connecting) {
      already_connected(NULL);
//      gnome_error_dialog("You are already connected to a server!  Disconnect before attempting to use this operation");
      return;
   }
   connect_cb();
}

void switch_page_cb(GtkWidget *widget, GtkNotebookPage *page, int page_num,
		    gpointer data) {
   GtkWidget *nth_page, *label;
   GtkStyle *old_style;
   
   if (page_num == 4 || widget == console_tab.notebook)
     gtk_widget_grab_focus(console_tab.entry);
   
   nth_page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(widget), page_num);
   label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(widget), nth_page);
   old_style = gtk_widget_get_style(label);
   if (old_style == gnapster_main.highlight)
     gtk_widget_set_style(label, gnapster_main.normal);
}

void build_list_cb() {
   if (building) {
      building = 0;
      return;
   }

   shared_list_handle(gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir)))), 
		      BUILD_LIST);
}

void update_list_cb() {
   shared_list_handle(gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir)))),
		      UPDATE_LIST);
}

void query_user_cb(GtkWidget *w, gpointer data) {
   gtk_notebook_set_page(GTK_NOTEBOOK(gnapster_main.notebook), 4);
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "/query ");
   gtk_widget_grab_focus(console_tab.entry);
}

void unquery_user_cb(GtkWidget *w, gpointer data) {
   if (!remove_console_tab(1))
     gnome_error_dialog("You must click on a user query tab before attempting to close it");
}

void menu_popup_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   GtkWidget *temp_menu;
   int row = -1, col;

   gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y,
				&row, &col);

   if (event->type == GDK_2BUTTON_PRESS && event->button == 1 &&
       widget == search_tab.clist) {
      Search *temp;
      Download *download;
      
      temp = gtk_clist_get_row_data(GTK_CLIST(widget), row);
      if (!temp) return;
      if (!temp->user) return;

      download = search_dup(temp, N_DOWNLOAD);
      handle_download(download, DOWNLOAD_NORMAL);
   } else if (event->type == GDK_2BUTTON_PRESS && event->button == 1 &&
	      widget == browse_tab.ctree) {
//      browse_download_cb(browse_tab.ctree, NULL);
   } else if (event->button == 3) {
      temp_menu = gnome_popup_menu_new(data);
      gnome_popup_menu_do_popup_modal(temp_menu, NULL, NULL, event, widget);
      gtk_widget_destroy(temp_menu);
   }
}

void console_activate_cb() {
   char *console_text = NULL, *command, *message;
   
   console_text = gtk_entry_get_text(GTK_ENTRY(console_tab.entry));
   if (!console_text || !conn_info.connected) return;
   if (!strlen(console_text)) return;
   
   if (loc_end > MAX_HISTORY) {
      GList *link;
      
      link = g_list_nth(console_history, 0);
      console_history = g_list_remove_link(console_history, link);
      d_free(link->data);
      loc_end--;
   }
   console_history = g_list_append(console_history, d_strdup(console_text));
   loc_end++;
   loc_curr = loc_end;

   if (console_text[0] != '/') {
      do_sendtext(NULL, d_strdup(console_text));
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
      return;
   }
   
   command = next_arg(console_text, &console_text);
   if (!command) {
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
      return;
   }
   message = last_arg(console_text, &console_text);

   if (!handle_command(command, message)) {
      gnapster_text_insert(CURR, SYSTEM,
			   "Unknown command (try \2/help\1)\n");
   }
   
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
}

int console_keypress_cb(GtkWidget *w, GdkEventKey *key) {
   GList *ptr;

   if (key->keyval >= GDK_1 && key->keyval <= GDK_9 && 
       key->state & GDK_MOD1_MASK) {
      gtk_notebook_set_page(GTK_NOTEBOOK(console_tab.notebook),
			    (key->keyval - 49));
      return 1;
   }
   
   if (key->keyval == GDK_Tab) {
      int page_num, x;
      ChannelInfo *chan;
      GList *nicks;
      GCompletion *comp;
      char *text, *lastword, *cnick, *strbuf, *npfx;
      
      gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key_press_event");
      
      page_num = gnapster_get_pagenum();
      
      if (page_num <= 1) 
	return 1;
      
      chan = g_list_nth_data(console_tab.channel_list,
			     page_num - 2);
      if (chan->user)
	return 1;
      
      text = gtk_entry_get_text(GTK_ENTRY(console_tab.entry));
      if (!strlen(text))
	return 1;
      
      lastword = strrchr(text, ' ');
      if (!lastword)
	lastword = text;
      else
	lastword++;
      
      if (lastword)
	lastword = d_strdup(all_lower(lastword));
      
      if (!lastword)
	return 1;
      
      comp = g_completion_new(nick_comp_func);
      g_completion_add_items(comp, GTK_CLIST(chan->clist)->row_list);
      
      nicks = g_completion_complete(comp, lastword, &npfx);
      if (npfx)
	d_hunk_add(npfx, strlen(npfx));
      x = strlen(lastword);
      d_free(lastword);
      if (!nicks || !npfx) {
	 g_completion_free(comp);
	 return 1;
      }
      
      cnick = ((User *)((GtkCListRow *)g_list_nth_data(nicks, 0))->data)->user;
      if (cnick && !strcmp(cnick, user_info.user) && g_list_length(nicks) > 1)
	cnick = ((User *)((GtkCListRow *)g_list_nth_data(nicks, 1))->data)->user;
      
      if (!cnick) {
	 d_free(npfx);
	 g_completion_free(comp);
	 return 1;
      }
      
      text[strlen(text) - x] = 0;
      
      strbuf = d_strdup_printf(DEBUG "%s%s ", text, cnick);
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), strbuf);
      d_free(strbuf);
      
      g_completion_free(comp);
      d_free(npfx);
      
      return 1;
   }
   
   if (key->keyval != GDK_Up && key->keyval != GDK_Down)
     return 0;
   
   switch(key->keyval) {
    case GDK_Up:
      loc_curr--;
      if (loc_curr < 0) loc_curr = 0;
      break;
    case GDK_Down:
      loc_curr++;
      if (loc_curr > loc_end) loc_curr = loc_end;
      break;
   }
   ptr = g_list_nth(console_history, loc_curr);
   if (ptr) {
      char *text;
      
      text = ptr->data;
      if (text)
	gtk_entry_set_text(GTK_ENTRY(w), text);
   } else if (loc_curr == loc_end) {
      gtk_entry_set_text(GTK_ENTRY(w), "");
   }
	
   gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key_press_event");
   
   return 1;
}

Search *get_data(const void *row) {
   return ((GtkCListRow *)row)->data;
}

User *get_chan_data(const void *row) {
   return ((GtkCListRow *)row)->data;
}

int compare_chan_user(GtkCList *clist, const void *row1, const void *row2) {
   User *s1 = get_chan_data(row1), *s2 = get_chan_data(row2);
   if (!s1 || !s2)
     return 0;
   
   return g_strcasecmp(s1->user, s2->user);
}

int compare_chan_conn(GtkCList *clist, const void *row1, const void *row2) {
   User *s1 = get_chan_data(row1), *s2 = get_chan_data(row2);
   if (!s1 || !s2)
     return 0;
   
   if (s1->conn <= s2->conn) return 1;
   if (s1->conn > s2->conn) return -1;
   
   return 0;
}

int compare_chan_shares(GtkCList *clist, const void *row1, const void *row2) {
   User *s1 = get_chan_data(row1), *s2 = get_chan_data(row2);
   if (!s1 || !s2) 
     return 0;
   
   if (s1->shares <= s2->shares) return 1;
   if (s1->shares > s2->shares) return -1;
   
   return 0;
}

void channel_sort(GtkCList *clist, int column) {
   void *sortfunc[3] = {
      compare_chan_user, compare_chan_shares, compare_chan_conn
   };
   
   if (column >= 3 || column < 0)
     return;
   
   gtk_clist_set_compare_func(clist, sortfunc[column]);
   
   if (clist->sort_column == column) {
      clist->sort_type = (clist->sort_type == GTK_SORT_ASCENDING) ? 
			 GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
   }
   
   gtk_clist_set_sort_column(clist, column);
   gtk_clist_sort(clist);
}

int compare_file(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1);
   Search *s2 = get_data(row2);
   
   return g_strcasecmp(s1->trunc_file, s2->trunc_file);
}

int compare_user(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1);
   Search *s2 = get_data(row2);
   
   return g_strcasecmp(s1->user, s2->user);
}

int compare_size(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1);
   Search *s2 = get_data(row2);
   
   if (s1->size <= s2->size) return 1;
   if (s1->size > s2->size) return -1;
   
   return 1;
}

int compare_length(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1);
   Search *s2 = get_data(row2);
   int seconds1, seconds2;
   
   seconds1 = (s1->size / ((s1->kbps * 1000) / 8));
   seconds2 = (s2->size / ((s2->kbps * 1000) / 8));
   if (seconds1 <= seconds2) return 1;
   if (seconds1 > seconds2) return -1;
   
   return 1;
}

int compare_bitrate(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1);
   Search *s2 = get_data(row2);
   
   if (s1->kbps <= s2->kbps) return 1;
   if (s1->kbps > s2->kbps) return -1;
   
   return 0;
}

int compare_connection(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1);
   Search *s2 = get_data(row2);
   
   if (s1->connection <= s2->connection) return 1;
   if (s1->connection > s2->connection) return -1;
   
   return 0;
}

void search_sort(GtkCList *clist, int column) {
   void *sortfunc[6] = {
      compare_file, compare_connection, compare_user, compare_size, 
      compare_length, compare_bitrate
   };
   
   if (column >= 6 || column < 0)
     return;

   gtk_clist_set_compare_func(clist, sortfunc[column]);

   if (clist->sort_column == column) {
      clist->sort_type = (clist->sort_type == GTK_SORT_ASCENDING) ?
		    	 GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
   }
   
   gtk_clist_set_sort_column(clist, column);
   
   gtk_clist_sort(clist);
}

void search_activate_cb() {
   char *not_string, *string;
   char extra_optns[1024], temp[1024];
   char *bps_table[] = {
      "96", "112", "128", "160", "192", NULL
   };
   
   if (!conn_info.connected || conn_info.searching) 
     return;
   
   string = gtk_entry_get_text(GTK_ENTRY(search_tab.gtk_entry));
   if (!string || string[0] == 0) 
     return;
   string = d_strdup(string);
   
   not_string = gtk_entry_get_text(GTK_ENTRY(search_tab.gtk_not_entry));
   /* yes this is safe ;) */
   not_string = d_strdup(not_string);

   memset(&extra_optns, 0, sizeof(extra_optns));
   gnapster_update_status(_("Searching..."), 1);
   conn_info.searching = 1;
   
   if (user_info.conf[LINE_SPEED_SWITCH]) {
      char *cmp, *ptr;
      
      cmp = d_strdup(listint_to_str(user_info.conf[LINE_SPEED_CMP], 0));
      ptr = cmp;
      while(*ptr) {
	 *ptr = toupper(*ptr);
	 ptr++;
      }
      
      g_snprintf(temp, sizeof(temp), " LINESPEED \"%s\" %i", cmp, user_info.conf[LINE_SPEED]);
      strcat(extra_optns, temp);
      d_free(cmp);
   }
   
   if (user_info.conf[BITRATE_SWITCH]) {
      char *cmp, *ptr, *bit;
      
      cmp = d_strdup(listint_to_str(user_info.conf[BITRATE_CMP], 0));
      ptr = cmp;
      while(*ptr) {
	 *ptr = toupper(*ptr);
	 ptr++;
      }

      bit = bps_table[user_info.conf[BITRATE]];
      g_snprintf(temp, sizeof(temp), " BITRATE \"%s\" \"%s\"", cmp, bit);
      strcat(extra_optns, temp);
      d_free(cmp);
   }
   
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(search_tab.local_only)))
     strcat(extra_optns, " LOCAL_ONLY");
   
   if (not_string[0]) {
      char buf[1024];
      
      g_snprintf(buf, sizeof(buf), " FILENAME EXCLUDES \"%s\"", not_string);
      strcat(extra_optns, buf);
   }

   g_strdown(string);
   napster_send(conn_info.sock, NAPSTER_SEARCH, "FILENAME CONTAINS \"%s\" MAX_RESULTS 100%s",
		string, extra_optns);
   
   d_free(string);
   d_free(not_string);

   gtk_clist_clear(GTK_CLIST(search_tab.clist));
   gtk_clist_freeze(GTK_CLIST(search_tab.clist));
   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 0);
}

void browse_user_menu(GtkWidget *clist) {
   void *temp;
   Type *n;
   int row = 0;
   char user[1024];
   
   if (!conn_info.connected) return;

   temp = gnapster_get_selection_data(clist, &row);
   if (!temp) return;
   
   n = (Type *)temp;
   
   if (n->type == N_SEARCH )
     g_snprintf(user, sizeof(user), ((Search *)temp)->user);
   else if (n->type == N_DOWNLOAD)
     g_snprintf(user, sizeof(user), ((Download *)temp)->user);
   else if (n->type == N_USER)
     g_snprintf(user, sizeof(user), ((User *)temp)->user);
   else 
     return;
   
   gnapster_update_status(_("Retrieving mp3 list..."), 1);
   conn_info.searching = 1;
   if (opennap_version(0, 34))
     napster_send(conn_info.sock, NAPSTER_OPENNAP_BROWSE, user);
   else
     napster_send(conn_info.sock, NAPSTER_BROWSE, user);
         
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
   gtk_clist_freeze(GTK_CLIST(browse_tab.ctree));
   gtk_entry_set_editable(GTK_ENTRY(browse_tab.entry), 0);
}

void browse_user_menu_cb(GtkWidget *w, gpointer data) {
   GtkWidget *data_widget;

   if (!data) return;

   data_widget = (GtkWidget *)data;
   browse_user_menu(data_widget);
}

void browse_user_cb() {
   char *string;
   
   if (GTK_CLIST(browse_tab.ctree)->freeze_count || !conn_info.connected ||
       conn_info.searching) 
     return;
   
   string = gtk_entry_get_text(GTK_ENTRY(browse_tab.entry));
   if (!strlen(string)) 
     return;
   
   gnapster_update_status(_("Retrieving mp3 list..."), 1);
   conn_info.searching = 1;
   if (opennap_version(0, 34))
     napster_send(conn_info.sock, NAPSTER_OPENNAP_BROWSE, string);
   else
     napster_send(conn_info.sock, NAPSTER_BROWSE, string);
   
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
   gtk_clist_freeze(GTK_CLIST(browse_tab.ctree));
   gtk_entry_set_editable(GTK_ENTRY(browse_tab.entry), 0);
}

void whois_user_menu_cb(GtkWidget *w, gpointer data) {
   GtkWidget *clist;
   int row = 0;
   Search *temp;
   Type *n;
   char *user;
   
   if (!data) 
     return;
   
   clist = (GtkWidget *)data;
   if (!clist) 
     return;
   
   temp = gnapster_get_selection_data(clist, &row);
   if (!temp) 
     return;
   
   n = (Type *)temp;
   
   if (n->type == N_SEARCH)
     user = temp->user;
   else if (n->type == N_USER) 
     user = ((User *)temp)->user;
   else if (n->type == N_DOWNLOAD)
     user = ((Download *)temp)->user;
   else
     return;

   gnapster_text_insert(CURR, SYSTEM, 
			"Sending whois request for \2%s\1...\n", user);
   napster_send(conn_info.sock, NAPSTER_WHOIS_REQUEST, user);
}

void connect_cb() {
   if (conn_info.connected || conn_info.connecting) return;
   if (!user_info.user) {
      gnapster_update_status(_("Please edit your preferences before attempting to connect!"), 0);
      return;
   }
   
   close(conn_info.sock);
   if (conn_info.sock_input) gdk_input_remove(conn_info.sock_input);
   conn_info.connected = 0;
   conn_info.connecting = 1;
   
   d_free(user_info.server);
   user_info.server = d_config_get_string("/gnapster/Options/server");
   user_info.server_port = gnome_config_get_int("/gnapster/Options/server_port");
   
   /* Napster servers changed their IP...this compensates... */
   if (!user_info.server || !strcmp(user_info.server, "208.184.216.222")) {
      gnome_config_set_string("/gnapster/Options/server", SERVER_NAPSTER_COM);
      gnome_config_set_int("/gnapster/Options/server_port", 8875);
      gnome_config_sync();
      user_info.server = d_config_get_string("/gnapster/Options/server");
      user_info.server_port = gnome_config_get_int("/gnapster/Options/server_port");
   }
   if (!user_info.server_port) {
      gnome_config_set_int("/gnapster/Options/server_port", 8875);
      gnome_config_sync();
      user_info.server_port = gnome_config_get_int("/gnapster/Options/server_port");
   }
   
   if (user_info.server_port == 8875) {
      /* Query server... */
      gnapster_update_status(_("Finding best host..."), 1);
      napster_get_best_host();
   } else {
      if (conn_info.ip)
	d_free(conn_info.ip);
      conn_info.ip = d_strdup(user_info.server);
      conn_info.port = user_info.server_port;
      conn_info.connecting = 1;
      conn_info.connected = 0;

      gnapster_update_status(_("Connecting to server..."), 1);
      napster_connect(conn_info.ip, conn_info.port);
   }
}

int reconnect_cb() {
   connect_cb();
   
   gtk_timeout_remove(conn_info.reconnect_timeout);
   
   return 0;
}

void disconnect_cb() {
   GList *ptr;
   
   if (!conn_info.connected && !conn_info.connecting) 
     return;
   
   close(conn_info.sock);
   if (conn_info.sock_input) 
     gdk_input_remove(conn_info.sock_input);
   if (conn_info.upload_input) 
     gdk_input_remove(conn_info.upload_input);
   if (conn_info.bind_sock_input) {
      gdk_input_remove(conn_info.bind_sock_input);
      close(conn_info.bind_sock);
   }
   conn_info.connected = 0;
   conn_info.connecting = 0;
   conn_info.searching = 0;
   conn_info.opennap_version = 0;
   
   gtk_label_set_text(GTK_LABEL(upload_tab.label), "");
   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 1);
   gtk_widget_set_sensitive(search_tab.media, 0);
   
   gtk_widget_hide(search_tab.not_label);
   gtk_widget_hide(search_tab.not_entry);
   
   gnapster_update_status(_("Disconnected"), 0);
   gtk_xtext_remove_lines(GTK_XTEXT(motd_tab.text), -1, TRUE);
   
   ptr = console_tab.channel_list;
   while(ptr) {
      if (((ChannelInfo *)ptr->data)->clist)
	gtk_clist_clear(GTK_CLIST(((ChannelInfo *)ptr->data)->clist));
      gtk_notebook_remove_page(GTK_NOTEBOOK(console_tab.notebook),
			       1);
      d_free(((ChannelInfo *)ptr->data)->channel_name);
      d_free(ptr->data);
      ptr = ptr->next;
   }
   g_list_free(console_tab.channel_list);
   console_tab.channel_list = NULL;
   
   conn_info.reconnect_timeout = gtk_timeout_add(60000 * 2, reconnect_cb,
						 NULL);
}

void search_download_cb(GtkWidget *w, gpointer data) {
   download_true_cb(search_tab.clist, data);
}

void browse_download_cb(GtkWidget *w, gpointer data) {
   download_true_cb(browse_tab.ctree, data);
}

void cancel_upload_cb() {
   Download *temp;
   GtkWidget *clist = (GtkWidget *)upload_tab.clist;
   
   temp = gnapster_get_selection_data(clist, NULL);
   if (!temp) return;
   
   upload_close(temp, TRANSFER_CANCEL);
}

char *listint_to_str(int i, int type) {
   char *cmp[] = { 
      "At least", "Equal to", "At best", NULL
   }, *conn[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL",
      "T1", "T3+", NULL
   };
   
   switch(type) {
    case 0:
      return cmp[i];
    case 1:
      return conn[i];
    default:
      return NULL;
   }
}

int liststr_to_int(char *s) {
   char *conn[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL",
      "T1", "T3+", NULL
   };
   char *bitrate[] = {
      "96", "112", "128", "160", "192", NULL
   };
   int i = 0;

   if (!strcmp(s, "At least"))
     return 0;
   else if (!strcmp(s, "Equal to"))
     return 1;
   else if (!strcmp(s, "At best"))
     return 2;
   
   while(conn[i]) {
      if (!strcmp(s, conn[i])) return i;
      i++;
   }
   i = 0;
   while(bitrate[i]) {
      if (!strcmp(s, bitrate[i])) return i;
      i++;
   }

   return 0;
}

void combo_changed(GtkWidget *widget, gpointer data) {
   WidgetProperty *prop;
   int val;
   
   prop = gtk_object_get_data(GTK_OBJECT(widget), "prop");
   if (!prop)
     return;
   
   if (GTK_IS_SPIN_BUTTON(widget))
     val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   else
     val = liststr_to_int(gtk_entry_get_text(GTK_ENTRY(widget)));
   user_info.conf[prop->key] = val;

   gnome_config_set_int(prop->conf, val);
   gnome_config_sync();
}

void preferences_changed_cb() {
   if (!prop_dlg.window) return;
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_dlg.window));
}

void preferences_button_cb(GtkWidget *widget, gint arg1, gpointer data) {
   char *user, *pass, *nap_port, *conn, *download_dir, *upload_dir;
   char *font;

   prop_dlg.calling++;
   if (prop_dlg.calling == 2) {
      prop_dlg.calling = 0;
      return;
   }
   
   user = gtk_entry_get_text(GTK_ENTRY(prop_dlg.user_entry));
   pass = gtk_entry_get_text(GTK_ENTRY(prop_dlg.pass_entry));
   nap_port = gtk_entry_get_text(GTK_ENTRY(prop_dlg.nap_port));
   conn = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(prop_dlg.connection)->entry));
   download_dir = gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.download_dir))));
   upload_dir = gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir))));
   
   font = gtk_entry_get_text(GTK_ENTRY(prop_dlg.font_entry));
   
//   server = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(prop_dlg.server_entry))));

   if (!strlen(user) || !strlen(pass) || !strlen(conn) || 
       !strlen(download_dir) || !strlen(nap_port) || !strlen(font)) {
      gnapster_update_status(_("Form not complete!  Make sure you fill in everything!"), 0);
      return;
   }
   
   user_info.connection = napster_conn_as_int(conn);
   if (conn_info.connected) {
      if (my_atoi(nap_port) != conn_info.nap_port ||
	  user_info.conf[FIREWALL] == 0) {
	 conn_info.nap_port = my_atoi(nap_port);
	 gdk_input_remove(conn_info.bind_sock_input);
	 close(conn_info.bind_sock);
	 if (conn_info.nap_port)
	   napster_bind(conn_info.nap_port);
	 if (conn_info.connected) {
	    napster_send(conn_info.sock, NAPSTER_CHANGE_PORT, "%i",
			 conn_info.nap_port);
	 }
	 gnome_config_set_int("/gnapster/Options/port", conn_info.nap_port);
      } else if (user_info.conf[FIREWALL] == 1) {
	 conn_info.nap_port = 0;
	 gdk_input_remove(conn_info.bind_sock_input);
	 close(conn_info.bind_sock);
	 if (conn_info.connected)
	   napster_send(conn_info.sock, NAPSTER_CHANGE_PORT, "0");
      }
   }

   gnome_config_set_string("/gnapster/User/user", user);
   gnome_config_set_string("/gnapster/User/pass", pass);
   gnome_config_set_int("/gnapster/User/connection", user_info.connection);
   gnome_config_set_string("/gnapster/User/dl_dir", download_dir);
   if (strlen(upload_dir)) 
     gnome_config_set_string("/gnapster/User/ul_dir", upload_dir);
   
   gnome_config_set_string("/gnapster/xtext/font", font);
   
   gnapster_config_set_int(0);
   
   d_free(user_info.user);
   d_free(user_info.pass);
   d_free(user_info.download_dir);
   d_free(user_info.upload_dir);
   user_info.user = d_config_get_string("/gnapster/User/user");
   user_info.pass = d_config_get_string("/gnapster/User/pass");
   user_info.download_dir = d_config_get_string("/gnapster/User/dl_dir");   
   user_info.upload_dir = d_config_get_string("/gnapster/User/ul_dir");
   
   if (!conn_info.connected && !conn_info.connecting)
     gnapster_update_status(_("Not connected (use the file menu to connect)"), -1);
}

void preferences_destroy_cb(GtkWidget *widget, gint arg1, gpointer data) {
   prop_dlg.window = NULL;
}

void clicked_cb(GtkWidget *widget, int *data) {
   WidgetProperty *prop;
   GList *ptr;
   
   *data = !*data;
   
   prop = gtk_object_get_data(GTK_OBJECT(widget), "prop");
   if (!prop)
     return;
   
   gnome_config_set_int(prop->conf, *data);
   ptr = prop->sensitive;
   while(ptr) {
      gtk_widget_set_sensitive(GTK_WIDGET(ptr->data), *data);
      ptr = ptr->next;
   }
   
   gnome_config_sync();
}

void preferences_clicked_cb(GtkWidget *widget, int *data) {
   clicked_cb(widget, data);
   preferences_changed_cb();
}

void preferences_cb() {
   if (prop_dlg.window) return;
   
   create_prop_dlg(preferences_changed_cb,
		   preferences_button_cb,
		   preferences_destroy_cb,
		   preferences_clicked_cb,
		   &(user_info.new_user));
   gtk_widget_show_all(prop_dlg.window);
}

void build_dlg_cancel_cb() {
   building = 0;
   gnome_dialog_close(GNOME_DIALOG(build_dlg.window));
   gdk_input_remove(build_dlg.pipe_input);
   if (build_dlg.cpid)
     kill(build_dlg.cpid, SIGUSR1);
}

void about_cb() {
   GtkWidget *about_box;
   const char *authors[] = { "jasta <jasta@gotlinux.org>", NULL };
   
   about_box = gnome_about_new(_("Gnapster"), VERSION,
			       _("Copyright (C) 1999-2000 Josh Guilfoyle"),
			       authors,
			       _("Napster (online mp3 community) client written for GNOME"),
			       NULL);
   gtk_widget_show(about_box);
}
