#ifndef __GNAPSTER_H
#define __GNAPSTER_H

#define NAP_VERSION "Gnapster 1.3.12"
/* Please note that this port will be held in ~/.gnome/gnapster upon
 * execution of Gnapster and this value will only be used when the config
 * file cannot be found */
#define NAP_PORT 6699

/* I don't think you really wanna do this unless you like pointless messages */
//#define GNAPSTER_DEBUG

#ifdef GNAPSTER_DEBUG
#define DPRINTF(x) do { printf x; } while(0)
#else
#define DPRINTF(x) do { ; } while(0)
#endif /* GNAPSTER_DEBUG */


#define MESSAGE ""
#define SYSTEM "\00314:\00310:\00311: \1"

#define DOWNLOAD_FORCE 1 << 0
#define DOWNLOAD_NORMAL 1 << 1

#define CURR -2
#define CONSOLE 0
#define MOTD 1

typedef struct {
   char *conf_str;
   int key;
   int conf_val;
} ConfigTable;

typedef void (*GnapsterFreeFunc) (void *data);
typedef struct {
   int type;
   GnapsterFreeFunc func;
   size_t size;
} FreeTable;

typedef struct {
   int type;
} Type;

typedef struct {
   char *channel_name;
   int user;
   GtkWidget *text;
   GtkWidget *clist;
   GtkWidget *vbox;
} ChannelInfo;

typedef struct {
   int sock;
   int sock_input;
   int bind_sock;
   int bind_sock_input;
   int upload_input;
   int napigator_sock;
   int napigator_input;
   struct sockaddr_in saddr;
   int timeout;
   int connecting;
   int connected;
   char *status;
   char *ip;
   int port;
   int nap_port;
   int searching;
   unsigned short opennap_version;
   int reconnect_timeout;
   int safe_exit;
} ConnInfo;

#define NAME_FIX 0
#define THEME_COLORS 1
#define AUTO_RM 2
#define OPENServer 3
#define AUTO_START_QUEUE 4
#define LINE_SPEED_SWITCH 5
#define LINE_SPEED_CMP 6
#define LINE_SPEED 7
#define BITRATE_SWITCH 8
#define BITRATE_CMP 9
#define BITRATE 10
#define MAX_UPLOADS_SWITCH 11
#define MAX_UPLOADS 12
#define REJECT_UL 13
#define FIREWALL 14
#define MAX_PERUSER_UPLOADS_SWITCH 15
#define MAX_PERUSER_UPLOADS 16
#define AUTOQUERY 17
#define MAX_DOWNLOADS_SWITCH 18
#define MAX_DOWNLOADS 19
#define MAX_PERUSER_DOWNLOADS_SWITCH 20
#define MAX_PERUSER_DOWNLOADS 21
#define HIDE_USERS 22
#define TERMINATION 23

typedef struct {
   char *user;
   char *pass;
   char *email;
   char *download_dir;
   char *upload_dir;
   int connection;
   int conf[TERMINATION + 1];
   int new_user;
   int sharing;
   double share_size;
   int queue;
   char *server;
   int server_port;
   char *default_font;
} UserInfo;

Download *gnapster_match_real_download(char *, char *, int *);
int upload_accepted(char *, char *, char *);
char *gnapster_sv();
void gnapster_config_combo_widget(GtkWidget *, char *);;
void gnapster_config_toggle_widget(GtkWidget *, char *, ...);
int verify_file_shared(char *);
FILE *open_file(Download *, char *);
GList *mp3_list_free(GList *);
void *download_dup(Download *);
void *search_dup(Search *, int);
Download *gnapster_parse_download(char *, int);
Search *gnapster_parse_search(char *);
Search *gnapster_parse_resume(char *);
void gnapster_fill_dl_resume_path(Download *);
void gnapster_fill_resume_path(Search *);
Search *gnapster_parse_browse(char *, char **, int);
int gnapster_update_appbar();
void gnapster_update_status(char *, int);
void gnapster_highlight(GtkWidget *, GtkWidget *);
int user_lookup(char *);
int gnapster_get_pagenum();
int gnapster_clist_append(GtkWidget *, GtkWidget *, void *, ...);
void *gnapster_get_selection_data(GtkWidget *, int *);
void gnapster_get_styles(GtkWidget *);
void gnapster_config_set_int(int);
void gnapster_config_get_int();

#endif /* __GNAPSTER_H */
